﻿using System;
using System.Text;
using System.Collections.Generic;
using Fortanix.SDKMS.Client;
using Fortanix.SDKMS.Model;
using Fortanix.SDKMS.Api;
using RestSharp;
using Newtonsoft.Json;

namespace test
{
    class Program
    {
        static void Main(string[] args) {
       
            // Extracting App UUID and Secret Key from API Key
            string apiKey = "";
            Configuration.Default.BasePath = "https://sdkms.fortanix.com";

            string decodedKey = Encoding.ASCII.GetString(Convert.FromBase64String(apiKey));
            string[] tokens = decodedKey.Split(':');
    	    Configuration.Default.Username = tokens[0];
            Configuration.Default.Password = tokens[1];

            // App Authentication
            Console.WriteLine("App Auth.....");
            AuthenticationApi authenticationApi = new AuthenticationApi();
            AuthResponse response = authenticationApi.Authorize();
    		Configuration.Default.AddApiKey("Authorization", response.AccessToken);
            Configuration.Default.AddApiKeyPrefix("Authorization", "Bearer");
    		
            // Create AES Key
            Console.WriteLine("Creating AES Key.....");
            SobjectRequest req = new SobjectRequest(Name: "TestAESKey1", ObjType: ObjectType.AES, KeySize: 128);
    		SecurityObjectsApi api = new SecurityObjectsApi();
    		KeyObject key = api.GenerateSecurityObject(req);

            // Encryption
            Console.WriteLine("Encryption.....");
    		EncryptRequest encReq = new EncryptRequest(Alg: ObjectType.AES, Mode: CryptMode.CBC, Plain: Encoding.ASCII.GetBytes("ABCD"));
    		EncryptionAndDecryptionApi encApi = new EncryptionAndDecryptionApi();
    		EncryptResponse encResp = encApi.Encrypt(key.Kid, encReq);
    		byte[] cipher = encResp.Cipher;
            Console.WriteLine("Cipher for plaintext: ABCD, is: " + Encoding.ASCII.GetString(cipher));
            
            // Decryption
            Console.WriteLine("Decryption.....");
            DecryptRequest decReq = new DecryptRequest(Alg: ObjectType.AES, Mode: CryptMode.CBC, Cipher: cipher, Iv: encResp.Iv);
            DecryptResponse decResp = encApi.Decrypt(key.Kid, decReq);
            byte[] plainBytes = decResp.Plain;
            string plain = Encoding.ASCII.GetString(plainBytes);
            Console.WriteLine("Decrypted Plaintext for Cipher is: " + plain);
            
            //Delete Key
            Console.WriteLine("Delete Key.....");
            api.DeleteSecurityObject(key.Kid);
            
            
            
            // Invoking Plugin. Currently PluginInvoke does not support by SDK directly.
            // Hence this will make direct REST call. Note: Plugin Invokation needs App Auth.
            String pluginId = "";
            RestClient restClient = Configuration.Default.ApiClient.RestClient;
            var request = new RestRequest("/sys/v1/plugins/" + pluginId, Method.POST);
            request.AddHeader("Authorization", Configuration.Default.GetApiKeyWithPrefix("Authorization"));
            request.AddHeader("Accept", "application/json");
            request.AddParameter("application/json", "{ Input json data }", ParameterType.RequestBody);

            IRestResponse result = (IRestResponse) restClient.Execute(request);
            Console.WriteLine("Plugin result: " + result.Content);
            
            
           
        }
    }
}
